#include <antiglut.h>
#include <assert.h>
#include <mvlib.h>
#include <stdio.h>
#include <../source/agmainloop.c>
#include <../source/mvlib.c>

#if 0

#include <../source/rg.c>

#define N	10000
vec3_t	vert[N];
ran_state_t state;

static void setup_scene()
{
	int i;
	vec3_t mins = {-3, -3, -3}, maxs = {3, 3, 3};
	ranInitState(&state, 1);
	for(i = 0;i < N;i++)
	{
		ranDirection(&state, vert[i]);
		vec3Scale(vert[i], vert[i], 3);
	}
	agSetSceneExtents(mins, maxs);
}    

vboolean draw_main(void)
{
	int i;
	glPointSize(1.8f);
	glColor4f(.8f,0.75f,0.72f,.6f);
	glBegin(GL_POINTS);
		for(i = 0;i < N;i++)
			glVertex3fv(vert[i]);
	glEnd();
	return true;
}

#else

/* 1st line segment */
vec3_t l1p1 = {-3, 2, 2}, l1p2 = {-2, -3, 0};
/* 2nd line segment */
vec3_t l2p1 = {3, 4, -2}, l2p2 = {-2, -1, 2};
/* bases of common vertical */
vec3_t cvp1, cvp2;

static void setup_scene()
{
	line3_t seg1, seg2;
	vec3_t mins, maxs;
	vec_t dist;
	/*
	 * Find bases of common vertical
	 */
	line3FromPoints(&seg1, l1p1, l1p2);
	line3FromPoints(&seg2, l2p1, l2p2);
	line3CommonVertical(&seg1, &seg2, cvp1, cvp2, &dist);
	printf("Distance between lines (not segments!): %f\n", dist);
	/*
	 * Compute bounging box of the scene
	 */
	bboxClear(mins, maxs);
	bboxAddPoint(mins, maxs, l1p1);
	bboxAddPoint(mins, maxs, l1p2);
	bboxAddPoint(mins, maxs, l2p1);
	bboxAddPoint(mins, maxs, l2p2);
	bboxAddPoint(mins, maxs, cvp1);
	bboxAddPoint(mins, maxs, cvp2);
	agSetSceneExtents(mins, maxs);
	/*
	 * Check whether cvp1->cvp2 orthogonal to both lines or not
	 */
	if(IsZero(dist))
	{
		puts("Distance near zero: common vertical of degenerate length.");
	}
	else
	{
#define CHECK_DOT(segname, v1, v2) \
		{ \
			vec_t dot_product = vec3Dot(v1, v2); \
			printf("Dot product for %s segment is %g; segments %s\n", \
				segname, dot_product, \
				IsZero(dot_product) ? "othogonal" : "non-orthogonal"); \
		}
		vec3_t cvdir;
		vec3Sub(cvdir, cvp2, cvp2);
		vec3Scale(cvdir, cvdir, 1/dist);
		CHECK_DOT("1st", cvdir, seg1.dir);
		CHECK_DOT("2nd", cvdir, seg2.dir);
	}
}

vboolean draw_main(void)
{
#define SOURCE_VERTICES() { \
		glVertex3fv(l1p1); \
		glVertex3fv(l1p2); \
		glVertex3fv(l2p1); \
		glVertex3fv(l2p2); \
	}

#define CV_VERTICES() { \
		glVertex3fv(cvp1); \
		glVertex3fv(cvp2); \
	}

	glPointSize(15);
	glColor4f(0, 1, 0, 0.4f);
	glBegin(GL_POINTS);
	SOURCE_VERTICES();
	glEnd();
	glLineWidth(4);
	glColor4f(0, 0.8f, 0, 0.7f);
	glBegin(GL_LINES);
	SOURCE_VERTICES();
	glEnd();
	
	glPointSize(20);
	glColor4f(0.7f, 0, 0.8f, 0.4f);
	glBegin(GL_POINTS);
	CV_VERTICES();
	glEnd();
	glLineWidth(8);
	glColor4f(0.5f, 0, 0.6f, 0.7f);
	glBegin(GL_LINES);
	CV_VERTICES();
	glEnd();


	return true;
}
	
#endif

int main()
{
	assert(sizeof(GLfloat) == sizeof(vec_t));
	agWindowConfig("Temporary test (use mouse/mwheel/arrows/numpad to control)",
		768, 768, GL_FALSE);
	if(!agInit(AG_REQUIRED_1_1,GL_FALSE,GL_FALSE,GL_FALSE))
		return -1;
	setup_scene();
	glClearColor(1,1,1,0.5f);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_POINT_SMOOTH);
	agMainLoop(draw_main, NULL);
	agShutdown();
	return 0;
}
